-- Title: Durability Helper
-- Author: JerichoHM
-- Version: 1.0.204
-- Desc: A simple addon for tracking equipment durability

-- GLOBALS
local dhFrameBG = { bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background.blp", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border.blp", tile = true, tileSize = 32, edgeSize = 16, insets = {left = 3, right = 3, top = 3, bottom = 3}}
local dhBarBG = { bgFile = "Interface\\PaperDollInfoFrame\\UI-Character-Skills-Bar", edgeFile = nil, tile = false, tileSize = 32, edgeSize = 0, insets = {left = 0, right = 0, top = 0, bottom = 0}}
local dhUnit = "player"
local y = -5
SLASH_DURABILITYHELPER1 = '/DHelper'

--Version Information
local Title = "Durability Helper"
local Core = "1"
local Revision = "0"
local Build = "204"

-- Frame Definitions
local dhFrame

-- Frame Tables
local itemFrames = {}

function dhMain()
	-- Create Core UI Frame
	dhFrame = CreateFrame("Frame", "dhFrame", UIParent)
	dhFrame:SetFrameStrata("LOW")
	dhFrame:SetPoint("CENTER", UIParent)
	dhFrame:SetWidth(60)
	dhFrame:SetHeight(208)
	dhFrame:SetBackdrop(dhFrameBG)

	--Mouse Enabling, Movement constraints, and Movment functions
	dhFrame:SetClampedToScreen(true)
	dhFrame:EnableMouse(true)
	dhFrame:SetMovable(true)
	dhFrame:RegisterForDrag("LeftButton")
	dhFrame:SetScript("OnDragStart", dhFrame.StartMoving)
	dhFrame:SetScript("OnDragStop", dhFrame.StopMovingOrSizing)

	-- Event Handlers
	dhFrame:RegisterEvent("ADDON_LOADED")
	dhFrame:RegisterEvent("UPDATE_INVENTORY_DURABILITY")
	dhFrame:SetScript("OnEvent", dhOnEvent)

	dhFrame:Show()
end

-- Primary (Core) functions for the addon below this point

function dhDrawItemFrame(itemSlot, slotID, slotTexture, slotCheckRelic, itemDurability, itemMaxDurability, itemTexture, itemStatus, itemIsBroken)
	local itemFrame = itemFrames[itemSlot]
	if not itemFrame then -- if frame doesn't exist
		-- The Item Frame
		itemFrame = CreateFrame("StatusBar", nil, dhFrame, nil)
		itemFrame:SetFrameStrata("LOW")
		itemFrame:SetPoint("TOPRIGHT", dhFrame, "TOPRIGHT", -5, y)
		itemFrame:SetSize(32,18)
		itemFrame:SetBackdrop(dhBarBG)
		itemFrame:SetBackdropColor(0.0, 0.7, 0.0, 0.5)
		itemFrame:SetStatusBarTexture("Interface\\PaperDollInfoFrame\\UI-Character-Skills-Bar")
		itemFrame:SetStatusBarColor(0.0, 0.7, 0.0, 1.0)
		itemFrame:SetMinMaxValues(0,1)
		itemFrame:SetValue(1)
		
		-- The Item Slot Icon
		local itemIcon = CreateFrame("Button", nil, itemFrame, nil)
		itemIcon:SetFrameStrata("LOW")
		itemIcon:SetPoint("TOPLEFT", itemFrame, "TOPLEFT", -18, 0)
		itemIcon:SetSize(18,18)
		if itemTexture then
			itemIconBG = { bgFile = itemTexture, edgeFile = nil, tile = false, tileSize = 18, edgeSize = 16, insets = {left = 0, right = 0, top = 0, bottom = 0}}
		else
			itemIconBG = { bgFile = slotTexture, edgeFile = nil, tile = false, tileSize = 18, edgeSize = 16, insets = {left = 0, right = 0, top = 0, bottom = 0}}
		end
		itemIcon:SetBackdrop(itemIconBG)
		itemFrame.btn1 = itemIcon
		
		-- The text for displaying the percentage of durability remaining
		local itemText = itemFrame:CreateFontString(nil, "ARTWORK")
		isValid = itemText:SetFont("Fonts\\FRIZQT__.TTF", 10)
		itemText:SetPoint("RIGHT", itemFrame, "RIGHT", -1, 1)
		itemFrame.text = itemText
		
		if itemDurability ~= nil then
			itemDurabilityPercentage = (itemDurability / itemMaxDurability) * 100
			itemFrame.text:SetFormattedText("|cFFFFFF00%i%%", itemDurabilityPercentage)
			itemFrame:SetBackdropColor(0.0, 0.7, 0.0, 0.5)
			itemFrame:SetStatusBarColor(0.0, 0.7, 0.0, 1.0)
		else
			itemFrame.text:SetFormattedText("|cFFFFFF00n/a")
		end
		itemFrames[itemSlot] = itemFrame
	else -- if Item Frame exists
		itemFrame:ClearAllPoints()
	end

	-- Updating code for existing frames
	
	if itemTexture then
		itemIconBG = { bgFile = itemTexture, edgeFile = nil, tile = false, tileSize = 18, edgeSize = 16, insets = {left = 0, right = 0, top = 0, bottom = 0}}
	else
		itemIconBG = { bgFile = slotTexture, edgeFile = nil, tile = false, tileSize = 18, edgeSize = 16, insets = {left = 0, right = 0, top = 0, bottom = 0}}
	end
	itemFrame.btn1:SetBackdrop(itemIconBG)
	
	if itemDurability ~= nil then
		itemFrame:SetValue(itemDurability / itemMaxDurability)
	else
		itemFrame:SetValue(1)
	end
	
	if itemDurability ~= nil then
		if itemStatus == 0 then
			itemDurabilityPercentage = (itemDurability / itemMaxDurability) * 100
			itemFrame.text:SetFormattedText("|cFFFFFF00%i%%", itemDurabilityPercentage)
			itemFrame:SetBackdropColor(0.0, 0.7, 0.0, 0.5)
			itemFrame:SetStatusBarColor(0.0, 0.7, 0.0, 1.0)
		elseif itemStatus == 1 or ((itemDurability / itemMaxDurability) * 100) == 0 then
			itemDurabilityPercentage = (itemDurability / itemMaxDurability) * 100
			itemFrame.text:SetFormattedText("|cFFFFFF00%i%%", itemDurabilityPercentage)
			itemFrame:SetBackdropColor(0.7, 0.7, 0.0, 0.5)
			itemFrame:SetStatusBarColor(0.7, 0.7, 0.0, 1.0)
		elseif itemStatus == 2 or ((itemDurability / itemMaxDurability) * 100) == 0 then
			itemDurabilityPercentage = (itemDurability / itemMaxDurability) * 100
			itemFrame.text:SetFormattedText("|cFFFFFF00%i%%", itemDurabilityPercentage)
			itemFrame:SetBackdropColor(0.7, 0.0, 0.0, 0.5)
			itemFrame:SetStatusBarColor(0.7, 0.0, 0.0, 1.0)
		end
	else
		itemFrame.text:SetFormattedText("|cFFFFFF00n/a")
	end
end

function dhUpdateData()
	-- Prepare slot id table
	local itemSlot = { "HeadSlot", "ShoulderSlot", "ChestSlot", "WristSlot", "MainHandSlot", "SecondaryHandSlot", "RangedSlot", "HandsSlot", "WaistSlot", "LegsSlot", "FeetSlot" }
	
	-- Gather important data per slot
	for k ,v in pairs(itemSlot) do
		local slotID, slotTexture, slotCheckRelic = GetInventorySlotInfo(v)
		local itemDurability, itemMaxDurability = GetInventoryItemDurability(slotID)
		local itemTexture = GetInventoryItemTexture(dhUnit, slotID)
		local itemStatus = GetInventoryAlertStatus(slotID)
		local itemIsBroken = GetInventoryItemBroken(dhUnit, slotID)
		
		dhDrawItemFrame(v, slotID, slotTexture, slotCheckRelic, itemDurability, itemMaxDurability, itemTexture, itemStatus, itemIsBroken, y)
		y = y - 18
	end
end

-- Primary (Core) functions for the addon above this point
-- Extra Functionality Functions below this point

function dhOnEvent(self, event)
	if event == "UPDATE_INVENTORY_DURABILITY" then
		dhUpdateData()
	elseif event == "ADDON_LOADED" then
		if dhLock == nil then
			dhLock = true
			dhFrame:EnableMouse(dhLock)
		elseif dhLock == true then
			dhFrame:EnableMouse(dhLock)
		elseif dhLock == false then
			dhFrame:EnableMouse(dhLock)
		end
	end
end

function SlashCmdList.DURABILITYHELPER(msg)
	if msg == "toggle" then
		dhToggle()
	elseif msg == "lock" then
		dhLocker()
	elseif msg == "reset" then
		dhReset()
	elseif msg == "info" then
		dhInfo()
	else
		print("Durability Helper Slash Commands")
		print("type /DHelper toggle to visibility")
		print("type /DHelper lock to toggle locking")
		print("type /DHelper reset to reset to default position")
		print("tpye /DHelper info to view current build information")
	end
end

function dhToggle()
	if dhFrame:IsVisible() == 1 then
		dhFrame:Hide()
		print("Durability Helper hidden")
	else
		dhFrame:Show()
		print("Durability Helper visible")
	end
end

function dhLocker()
	-- Remember dhLock is backwards. true for unlocked and false for locked
	if dhLock == true then
		dhLock = false
		dhFrame:EnableMouse(dhLock)
		print("Durability Helper locked")
	elseif dhLock == false then
		dhLock = true
		dhFrame:EnableMouse(dhLock)
		print("Durability Helper unlocked")
	end
end

function dhReset()
	dhFrame:SetPoint("CENTER", UIParent, "CENTER")
	dhLock = true
	dhFrame:EnableMouse(dhLock)
	dhFrame:Show()
	print("Durability Helper Reset")
end

function dhInfo()
	print(Title .. " " .. Core .. "." .. Revision .. "." .. Build)
end

dhMain()